@@ -4,36 +4,44 @@ Common solar physics coordinate systems.
 This submodule implements various solar physics coordinate frames for use with
 the `astropy.coordinates` module.
 """
-from __future__ import absolute_import, division
-
 import numpy as np
-from astropy import units as u
+
+import astropy.units as u
 from astropy.coordinates import Attribute, ConvertError
-from astropy.coordinates.baseframe import (BaseCoordinateFrame,
-                                           RepresentationMapping)
-from astropy.coordinates.representation import (CartesianRepresentation,
+from astropy.coordinates.baseframe import BaseCoordinateFrame, RepresentationMapping
+from astropy.coordinates.representation import (CartesianRepresentation, SphericalRepresentation,
                                                 CylindricalRepresentation,
-                                                SphericalRepresentation,
                                                 UnitSphericalRepresentation)
-from sunpy import sun
 
-from .frameattributes import (ObserverCoordinateAttribute,
-                              TimeFrameAttributeSunPy)
+from sunpy.sun.constants import radius as _RSUN
 
-try:
-    from astropy.units import allclose as quantity_allclose
-except ImportError:
-    from astropy.tests.helper import quantity_allclose
+from .frameattributes import TimeFrameAttributeSunPy, ObserverCoordinateAttribute
+
+__all__ = ['HeliographicStonyhurst', 'HeliographicCarrington',
+           'Heliocentric', 'Helioprojective']
 
 
-RSUN_METERS = sun.constants.get('radius').si.to(u.m)
-DSUN_METERS = sun.constants.get('mean distance').si.to(u.m)
+class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
+    """
+    Inject a nice way of representing the object which the coordinate represents.
+    """
 
-__all__ = ['HeliographicStonyhurst', 'HeliographicCarrington', 'Heliocentric',
-           'Helioprojective']
+    def __init__(self, *args, **kwargs):
+        self.object_name = None
+        return super().__init__(*args, **kwargs)
+
+    def __str__(self):
+        """
+        We override this here so that when you print a SkyCoord it shows the
+        observer as the string and not the whole massive coordinate.
+        """
+        if getattr(self, "object_name", None):
+            return f"<{self.__class__.__name__} Coordinate for '{self.object_name}'>"
+        else:
+            return super().__str__()
 
 
-class HeliographicStonyhurst(BaseCoordinateFrame):
+class HeliographicStonyhurst(SunPyBaseCoordinateFrame):
     """
     A coordinate or frame in the Stonyhurst Heliographic system.
 
@@ -47,7 +55,7 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
 
         HeliographicStonyhurst(lon, lat, obstime)
         HeliographicStonyhurst(lon, lat, radius, obstime)
-        HeliographicStonyhurst(x, y, z, obstime, representation='cartesian')
+        HeliographicStonyhurst(x, y, z, obstime, representation_type='cartesian')
 
     Parameters
     ----------
@@ -81,15 +89,15 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
     ...               frame="heliographic_stonyhurst",
     ...               obstime="2010/01/01T00:00:45")
     >>> sc
-    <SkyCoord (HeliographicStonyhurst: obstime=2010-01-01 00:00:45): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicStonyhurst: obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
         (1., 1., 2.)>
     >>> sc.frame
-    <HeliographicStonyhurst Coordinate (obstime=2010-01-01 00:00:45): (lon, lat, radius) in (deg, deg, km)
+    <HeliographicStonyhurst Coordinate (obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
         (1., 1., 2.)>
     >>> sc = SkyCoord(HeliographicStonyhurst(-10*u.deg, 2*u.deg))
     >>> sc
     <SkyCoord (HeliographicStonyhurst: obstime=None): (lon, lat, radius) in (deg, deg, km)
-        (-10., 2., 695508.)>
+        (-10., 2., 695700.)>
 
     Notes
     -----
@@ -122,12 +130,12 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
     _default_wrap_angle = 180*u.deg
 
     def __init__(self, *args, **kwargs):
-        _rep_kwarg = kwargs.get('representation', None)
+        _rep_kwarg = kwargs.get('representation_type', None)
         wrap = kwargs.pop('wrap_longitude', True)
 
         if ('radius' in kwargs and kwargs['radius'].unit is u.one and
-                quantity_allclose(kwargs['radius'], 1*u.one)):
-            kwargs['radius'] = RSUN_METERS.to(u.km)
+                u.allclose(kwargs['radius'], 1*u.one)):
+            kwargs['radius'] = _RSUN.to(u.km)
 
         super(HeliographicStonyhurst, self).__init__(*args, **kwargs)
 
@@ -135,9 +143,9 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
         # If representation was explicitly passed, do not change the rep.
         if not _rep_kwarg:
             # If we were passed a 3D rep extract the distance, otherwise
-            # calculate it from RSUN.
+            # calculate it from _RSUN.
             if isinstance(self._data, UnitSphericalRepresentation):
-                distance = RSUN_METERS.to(u.km)
+                distance = _RSUN.to(u.km)
                 self._data = SphericalRepresentation(lat=self._data.lat,
                                                      lon=self._data.lon,
                                                      distance=distance)
@@ -186,13 +194,13 @@ class HeliographicCarrington(HeliographicStonyhurst):
     ...               frame="heliographic_carrington",
     ...               obstime="2010/01/01T00:00:30")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01 00:00:30): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:30.000): (lon, lat, radius) in (deg, deg, km)
         (1., 2., 3.)>
 
     >>> sc = SkyCoord([1,2,3]*u.deg, [4,5,6]*u.deg, [5,6,7]*u.km,
     ...               obstime="2010/01/01T00:00:45", frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01 00:00:45): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
         [(1., 4., 5.), (2., 5., 6.), (3., 6., 7.)]>
     """
 
@@ -266,15 +274,13 @@ class Heliocentric(BaseCoordinateFrame):
     >>> sc = SkyCoord(CartesianRepresentation(10*u.km, 1*u.km, 2*u.km),
     ...               obstime="2011/01/05T00:00:50", frame="heliocentric")
     >>> sc
-    <SkyCoord (Heliocentric: obstime=2011-01-05 00:00:50, observer=<HeliographicStonyhurst Coordinate (obstime=2011-01-05 00:00:50): (lon, lat, radius) in (deg, deg, AU)
-        (0., -3.43939103, 0.98334411)>): (x, y, z) in km
+    <SkyCoord (Heliocentric: obstime=2011-01-05T00:00:50.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (x, y, z) in km
         (10., 1., 2.)>
 
     >>> sc = SkyCoord([1,2]*u.km, [3,4]*u.m, [5,6]*u.cm, frame="heliocentric", obstime="2011/01/01T00:00:54")
 
     >>> sc
-    <SkyCoord (Heliocentric: obstime=2011-01-01 00:00:54, observer=<HeliographicStonyhurst Coordinate (obstime=2011-01-01 00:00:54): (lon, lat, radius) in (deg, deg, AU)
-        (0., -2.97725356, 0.98335586)>): (x, y, z) in (km, m, cm)
+    <SkyCoord (Heliocentric: obstime=2011-01-01T00:00:54.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (x, y, z) in (km, m, cm)
         [(1., 3., 5.), (2., 4., 6.)]>
     """
 
@@ -288,7 +294,7 @@ class Heliocentric(BaseCoordinateFrame):
     observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
 
 
-class Helioprojective(BaseCoordinateFrame):
+class Helioprojective(SunPyBaseCoordinateFrame):
     """
     A coordinate or frame in the Helioprojective (Cartesian) system.
 
@@ -310,8 +316,10 @@ class Helioprojective(BaseCoordinateFrame):
     obstime: SunPy Time
         The date and time of the observation, used to convert to heliographic
         carrington coordinates.
-    observer: `~sunpy.coordinates.frames.HeliographicStonyhurst`
-        The coordinate of the observer in the solar system.
+    observer: `~sunpy.coordinates.frames.HeliographicStonyhurst`, str
+        The coordinate of the observer in the solar system. If you supply a string,
+        it must be a solar system body that can be parsed by
+        `~sunpy.coordinates.ephemeris.get_body_heliographic_stonyhurst`.
     rsun: `~astropy.units.Quantity`
         The physical (length) radius of the Sun. Used to calculate the position
         of the limb for calculating distance from the observer to the
@@ -325,13 +333,11 @@ class Helioprojective(BaseCoordinateFrame):
     >>> sc = SkyCoord(0*u.deg, 0*u.deg, 5*u.km, obstime="2010/01/01T00:00:00",
     ...               frame="helioprojective")
     >>> sc
-    <SkyCoord (Helioprojective: obstime=2010-01-01 00:00:00, rsun=695508.0 km, observer=<HeliographicStonyhurst Coordinate (obstime=2010-01-01 00:00:00): (lon, lat, radius) in (deg, deg, AU)
-        (0., -3.00724817, 0.98330294)>): (Tx, Ty, distance) in (arcsec, arcsec, km)
+    <SkyCoord (Helioprojective: obstime=2010-01-01T00:00:00.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (Tx, Ty, distance) in (arcsec, arcsec, km)
         (0., 0., 5.)>
     >>> sc = SkyCoord(0*u.deg, 0*u.deg, obstime="2010/01/01T00:00:00", frame="helioprojective")
     >>> sc
-    <SkyCoord (Helioprojective: obstime=2010-01-01 00:00:00, rsun=695508.0 km, observer=<HeliographicStonyhurst Coordinate (obstime=2010-01-01 00:00:00): (lon, lat, radius) in (deg, deg, AU)
-        (0., -3.00724817, 0.98330294)>): (Tx, Ty) in arcsec
+    <SkyCoord (Helioprojective: obstime=2010-01-01T00:00:00.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (Tx, Ty) in arcsec
         (0., 0.)>
     """
 
@@ -357,7 +363,7 @@ class Helioprojective(BaseCoordinateFrame):
     }
 
     obstime = TimeFrameAttributeSunPy()
-    rsun = Attribute(default=RSUN_METERS.to(u.km))
+    rsun = Attribute(default=_RSUN.to(u.km))
     observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
     _default_wrap_angle = 180*u.deg
 
@@ -385,7 +391,7 @@ class Helioprojective(BaseCoordinateFrame):
         """
         # Skip if we already are 3D
         if (isinstance(self._data, SphericalRepresentation) and
-                not (self.distance.unit is u.one and quantity_allclose(self.distance, 1*u.one))):
+                not (self.distance.unit is u.one and u.allclose(self.distance, 1*u.one))):
             return self
 
         if not isinstance(self.observer, BaseCoordinateFrame):
@@ -398,7 +404,9 @@ class Helioprojective(BaseCoordinateFrame):
         alpha = np.arccos(np.cos(lat) * np.cos(lon)).to(lat.unit)
         c = self.observer.radius**2 - self.rsun**2
         b = -2 * self.observer.radius * np.cos(alpha)
-        d = ((-1*b) - np.sqrt(b**2 - 4*c)) / 2
+        # Ingore sqrt of NaNs
+        with np.errstate(invalid='ignore'):
+            d = ((-1*b) - np.sqrt(b**2 - 4*c)) / 2
 
         return self.realize_frame(SphericalRepresentation(lon=lon,
                                                           lat=lat,
