@@ -1,11 +1,6 @@
-from distutils.version import LooseVersion
-
 import numpy as np
 import pytest
 
-from  astropy import __version__
-ASTROPY_LT_3 = LooseVersion(__version__) < LooseVersion('3')
-
 import astropy.units as u
 from astropy.tests.helper import quantity_allclose, assert_quantity_allclose
 from astropy.coordinates import (SkyCoord, get_body_barycentric, Angle,
@@ -19,7 +14,8 @@ from astropy.time import Time
 
 from sunpy.coordinates import (Helioprojective, HeliographicStonyhurst,
                                HeliographicCarrington, Heliocentric,
-                               get_sun_L0, get_earth)
+                               get_earth)
+from sunpy.coordinates import sun
 from sunpy.time import parse_time
 
 
@@ -99,14 +95,14 @@ def test_hpc_hpc_null():
 
 def test_hcrs_hgs():
     # Get the current Earth location in HCRS
-    adate = Time(parse_time('2015/05/01 01:13:00'))
+    adate = parse_time('2015/05/01 01:13:00')
     earth_hcrs = SkyCoord(get_body_barycentric('earth', adate), frame='icrs', obstime=adate).hcrs
 
     # Convert from HCRS to HGS
     earth_hgs = earth_hcrs.transform_to(HeliographicStonyhurst)
 
     # The HGS longitude of the Earth should be zero within numerical error
-    # Due to an issue with wrapping at +-360, we shift it to pass the test
+    # Due to an issue with wrapping at +-360, we shift it to pass the test.
     assert quantity_allclose((earth_hgs.lon+1*u.deg) % (360*u.deg), 1*u.deg, atol=1e-12*u.deg)
 
     # The HGS latitude and radius should be within valid ranges
@@ -162,7 +158,7 @@ def test_hgs_hgc_roundtrip():
     hgcout = hgsin.transform_to(HeliographicCarrington(obstime=obstime))
 
     assert_quantity_allclose(hgsin.lat, hgcout.lat)
-    assert_quantity_allclose(hgcout.lon, get_sun_L0(obstime))
+    assert_quantity_allclose(hgcout.lon, sun.L0(obstime))
 
     hgsout = hgcout.transform_to(HeliographicStonyhurst(obstime=obstime))
 
@@ -176,16 +172,11 @@ def test_hgs_cartesian_rep_to_hpc():
     # spherical representation
 
     obstime = "2011-01-01"
-    if ASTROPY_LT_3:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation='cartesian')
-    else:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation_type='cartesian')
+    hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
+                             frame=HeliographicStonyhurst(obstime=obstime),
+                             representation_type='cartesian')
     hgscoord_sph = hgscoord_cart.copy()
-    hgscoord_sph.representation = 'spherical'
+    hgscoord_sph.representation_type = 'spherical'
     hpccoord_cart = hgscoord_cart.transform_to(Helioprojective(obstime=obstime))
     hpccoord_sph = hgscoord_sph.transform_to(Helioprojective(obstime=obstime))
     assert_quantity_allclose(hpccoord_cart.Tx, hpccoord_sph.Tx)
@@ -199,16 +190,11 @@ def test_hgs_cartesian_rep_to_hcc():
     # spherical representation
 
     obstime = "2011-01-01"
-    if ASTROPY_LT_3:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation='cartesian')
-    else:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation_type='cartesian')
+    hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
+                             frame=HeliographicStonyhurst(obstime=obstime),
+                             representation_type='cartesian')
     hgscoord_sph = hgscoord_cart.copy()
-    hgscoord_sph.representation = 'spherical'
+    hgscoord_sph.representation_type = 'spherical'
     hcccoord_cart = hgscoord_cart.transform_to(Heliocentric(obstime=obstime))
     hcccoord_sph = hgscoord_sph.transform_to(Heliocentric(obstime=obstime))
     assert_quantity_allclose(hcccoord_cart.x, hcccoord_sph.x)
@@ -222,16 +208,11 @@ def test_hgs_cartesian_rep_to_hgc():
     # spherical representation
 
     obstime = "2011-01-01"
-    if ASTROPY_LT_3:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation='cartesian')
-    else:
-        hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
-                                 frame=HeliographicStonyhurst(obstime=obstime),
-                                 representation_type='cartesian')
+    hgscoord_cart = SkyCoord(x=1*u.km, y=0.*u.km, z=0.*u.km,
+                             frame=HeliographicStonyhurst(obstime=obstime),
+                             representation_type='cartesian')
     hgscoord_sph = hgscoord_cart.copy()
-    hgscoord_sph.representation = 'spherical'
+    hgscoord_sph.representation_type = 'spherical'
     # HGC
     hgccoord_cart = hgscoord_cart.transform_to(HeliographicCarrington(obstime=obstime))
     hgccoord_sph = hgscoord_sph.transform_to(HeliographicCarrington(obstime=obstime))
