@@ -2,20 +2,17 @@
 
 Author: `Keith Hughitt <keith.hughitt@nasa.gov>`
 """
-from __future__ import absolute_import, print_function, division
-
 import numpy as np
 
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 
 import astropy.units as u
 
 from sunpy.map import GenericMap
-from sunpy.visualization import axis_labels_from_ctype
+from sunpy.visualization import axis_labels_from_ctype, peek_show
 
 from sunpy.util import expand_list
-from sunpy.extern import six
-from sunpy.extern.six.moves import range
 
 __all__ = ['CompositeMap']
 
@@ -58,7 +55,7 @@ class CompositeMap(object):
         Set the plot settings for a map with the CompositeMap.
     set_zorder(index, zorder)
         Set the layering preference (z-order) for a map within the CompositeMap.
-    plot(figure=None, overlays=None, draw_limb=False, gamma=1.0,
+    plot(figure=None, overlays=None, draw_limb=False,
     draw_grid=False, colorbar=True, basic_plot=False,title="SunPy Plot",
     matplot_args)
         Plots the composite map object using matplotlib
@@ -66,12 +63,12 @@ class CompositeMap(object):
     Examples
     --------
     >>> import sunpy.map
-    >>> import sunpy.data.sample
+    >>> import sunpy.data.sample  # doctest: +REMOTE_DATA
     >>> comp_map = sunpy.map.Map(sunpy.data.sample.AIA_171_IMAGE,
     ...                          sunpy.data.sample.EIT_195_IMAGE,
-    ...                          composite=True)
-    >>> comp_map.add_map(sunpy.map.Map(sunpy.data.sample.RHESSI_IMAGE))
-    >>> comp_map.peek()   # doctest: +SKIP
+    ...                          composite=True)  # doctest: +REMOTE_DATA
+    >>> comp_map.add_map(sunpy.map.Map(sunpy.data.sample.RHESSI_IMAGE))  # doctest: +REMOTE_DATA
+    >>> comp_map.peek()  # doctest: +SKIP
 
     """
     def __init__(self, *args, **kwargs):
@@ -223,7 +220,9 @@ class CompositeMap(object):
             The index of the map in the composite map.
 
         alpha : `float`
-            A float in the range 0 to 1.
+            A float in the range 0 to 1.  Increasing values of alpha decrease
+            the transparency of the layer (0 is complete transparency, 1
+            indicates the layer will be completely opaque).
 
         Returns
         -------
@@ -328,12 +327,13 @@ class CompositeMap(object):
 
         index_check = hasattr(self._maps[index], 'rsun_obs')
         if not index_check or index is None:
-            raise ValueError("Specified index does not have all the required attributes to draw limb.")
+            raise ValueError("Specified index does not have all"
+                             " the required attributes to draw limb.")
 
         return self._maps[index].draw_limb(axes=axes, **kwargs)
 
-    @u.quantity_input(grid_spacing=u.deg)
-    def draw_grid(self, index=None, axes=None, grid_spacing=20*u.deg, **kwargs):
+    @u.quantity_input
+    def draw_grid(self, index=None, axes=None, grid_spacing: u.deg=20*u.deg, **kwargs):
         """Draws a grid over the surface of the Sun.
 
         Parameters
@@ -365,7 +365,8 @@ class CompositeMap(object):
 
         index_check = all([hasattr(self._maps[index], k) for k in needed_attrs])
         if not index_check or index is None:
-            raise ValueError("Specified index does not have all the required attributes to draw grid.")
+            raise ValueError("Specified index does not have all"
+                             " the required attributes to draw grid.")
 
         ax = self._maps[index].draw_grid(axes=axes, grid_spacing=grid_spacing, **kwargs)
         return ax
@@ -414,9 +415,13 @@ class CompositeMap(object):
         # Plot layers of composite map
         for m in self._maps:
             # Parameters for plotting
+            bl = m._get_lon_lat(m.bottom_left_coord)
+            tr = m._get_lon_lat(m.top_right_coord)
+            x_range = list(u.Quantity([bl[0], tr[0]]).to(m.spatial_units[0]).value)
+            y_range = list(u.Quantity([bl[1], tr[1]]).to(m.spatial_units[1]).value)
             params = {
                 "origin": "lower",
-                "extent": list(m.xrange.value) + list(m.yrange.value),
+                "extent": x_range + y_range,
                 "cmap": m.plot_settings['cmap'],
                 "norm": m.plot_settings['norm'],
                 "alpha": m.alpha,
@@ -450,9 +455,13 @@ class CompositeMap(object):
         plt.sci(ret[0])
         return ret
 
+    @peek_show
     def peek(self, colorbar=True, basic_plot=False, draw_limb=True,
              draw_grid=False, **matplot_args):
-        """Displays the map in a new figure.
+        """
+        Displays a graphical overview of the data in this object for user evaluation.
+        For the creation of plots, users should instead use the `~sunpy.map.CompositeMap.plot`
+        method and Matplotlib's pyplot framework.
 
         Parameters
         ----------
@@ -500,12 +509,12 @@ class CompositeMap(object):
             if draw_grid:
                 self.draw_grid(axes=axes)
 
-        elif isinstance(draw_grid, six.integer_types + (float,)):
+        elif isinstance(draw_grid, (int, float)):
             self.draw_grid(axes=axes, grid_spacing=draw_grid)
         else:
             raise TypeError("draw_grid should be bool, int, long or float")
 
-        figure.show()
+        return figure
 
 
 class OutOfRangeAlphaValue(ValueError):
