@@ -1,13 +1,12 @@
 """Hinode XRT and SOT Map subclass definitions"""
-from __future__ import absolute_import, print_function, division
 #pylint: disable=W0221,W0222,E1101,E1121
 
 __author__ = ["Jack Ireland, Jose Ivan Campos-Rozo, David Perez-Suarez"]
 __email__ = "jack.ireland@nasa.gov"
 
-import numpy as np
+import matplotlib.pyplot as plt
+
 from sunpy.map import GenericMap
-from sunpy.cm import cm
 
 __all__ = ['XRTMap', 'SOTMap']
 
@@ -65,7 +64,7 @@ class XRTMap(GenericMap):
         self.meta['detector'] = "XRT"
 #        self.meta['instrume'] = "XRT"
         self.meta['telescop'] = "Hinode"
-        self.plot_settings['cmap'] = cm.get_cmap(name='hinodexrt')
+        self.plot_settings['cmap'] = plt.get_cmap(name='hinodexrt')
 
     @property
     def measurement(self):
@@ -131,7 +130,7 @@ class SOTMap(GenericMap):
                  'SOT/SP': 'intensity',  # For the 1st 2 dimensions
                  }
 
-        self.plot_settings['cmap'] = cm.get_cmap('hinodesot' + color[self.instrument])
+        self.plot_settings['cmap'] = plt.get_cmap('hinodesot' + color[self.instrument])
 
     @classmethod
     def is_datasource_for(cls, data, header, **kwargs):
