@@ -1,26 +1,29 @@
 import os
 import copy
-import tempfile
+import pathlib
+from unittest import mock
 
-import pytest
 import hypothesis.strategies as st
-from hypothesis import given, assume, example, settings
+import pytest
+from drms import DrmsQueryError
+from hypothesis import assume, given, settings
+from parfive import Results
+from parfive.utils import FailedDownload
 
 import astropy.units as u
-from drms import DrmsQueryError
 
-from sunpy.net import attr
-from sunpy.net.vso import attrs as va
-from sunpy.net import Fido, attrs as a
-from sunpy.net.vso import QueryResponse as vsoQueryResponse
-from sunpy.net.fido_factory import DownloadResponse, UnifiedResponse
-from sunpy.net.dataretriever.client import CLIENTS, QueryResponse
-from sunpy.util.datatype_factory_base import NoMatchError, MultipleMatchError
-from sunpy.time import TimeRange, parse_time
 from sunpy import config
-
-from sunpy.net.tests.strategies import (online_instruments, offline_instruments,
-                                        time_attr, range_time, goes_time)
+from sunpy.net import Fido, attr
+from sunpy.net import attrs as a
+from sunpy.net.base_client import BaseClient
+from sunpy.net.dataretriever.client import QueryResponse
+from sunpy.net.dataretriever.sources.goes import XRSClient
+from sunpy.net.fido_factory import UnifiedResponse
+from sunpy.net.tests.strategies import goes_time, offline_instruments, online_instruments, time_attr
+from sunpy.net.vso import QueryResponse as vsoQueryResponse
+from sunpy.net.vso.vso import DownloadFailed
+from sunpy.time import TimeRange, parse_time
+from sunpy.util.datatype_factory_base import MultipleMatchError
 
 TIMEFORMAT = config.get("general", "time_format")
 
@@ -32,8 +35,6 @@ def offline_query(draw, instrument=offline_instruments()):
     """
     query = draw(instrument)
     # If we have AttrAnd then we don't have GOES
-    if isinstance(query, a.Instrument) and query.value == 'norh':
-        query &= a.Wavelength(17*u.GHz)
     if isinstance(query, a.Instrument) and query.value == 'goes':
         query &= draw(goes_time())
     else:
@@ -42,16 +43,14 @@ def offline_query(draw, instrument=offline_instruments()):
 
 
 @st.composite
-def online_query(draw, instrument=online_instruments(), time=time_attr()):
+def online_query(draw, instrument=online_instruments()):
     query = draw(instrument)
-    # If we have AttrAnd then we don't have RHESSI
-    if isinstance(query, a.Instrument) and query.value == 'rhessi':
-        # Build a time attr which does not span a month.
-        year = draw(st.integers(min_value=2010, max_value=2017))
-        month = draw(st.integers(min_value=3, max_value=9))
-        days = draw(st.integers(min_value=10, max_value=25))
-        query = query & a.Time("{}-{}-01".format(year, month, days),
-                               "{}-{}-{}".format(year, month, days))
+
+    if isinstance(query, a.Instrument) and query.value == 'eve':
+        query &= a.Level(0)
+    if isinstance(query, a.Instrument) and query.value == 'norh':
+        query &= a.Wavelength(17*u.GHz)
+
     return query
 
 
@@ -62,9 +61,15 @@ def test_offline_fido(query):
     check_response(query, unifiedresp)
 
 
-@settings(deadline=50000)
 @pytest.mark.remote_data
-@given(online_query())
+@pytest.mark.flaky(reruns=5)
+# Until we get more mocked, we can't really do this to online clients.
+# TODO: Hypothesis this again
+@pytest.mark.parametrize("query", [
+    (a.Instrument('eve') & a.Time('2014/7/7', '2014/7/14') & a.Level(0)),
+    (a.Instrument('rhessi') & a.Time('2014/7/7', '2014/7/14')),
+    (a.Instrument('norh') & a.Time('2014/7/7', '2014/7/14') & a.Wavelength(17*u.GHz)),
+])
 def test_online_fido(query):
     unifiedresp = Fido.search(query)
     check_response(query, unifiedresp)
@@ -92,29 +97,28 @@ def check_response(query, unifiedresp):
 
 
 @pytest.mark.remote_data
-def test_save_path():
+def test_save_path(tmpdir):
     qr = Fido.search(a.Instrument('EVE'), a.Time("2016/10/01", "2016/10/02"), a.Level(0))
 
     # Test when path is str
-    with tempfile.TemporaryDirectory() as target_dir:
-        files = Fido.fetch(qr, path=os.path.join(target_dir, "{instrument}", "{level}"))
-        for f in files:
-            assert target_dir in f
-            assert "eve{}0".format(os.path.sep) in f
+    files = Fido.fetch(qr, path=str(tmpdir / "{instrument}" / "{level}"))
+    for f in files:
+        assert str(tmpdir) in f
+        assert "eve{}0".format(os.path.sep) in f
 
 
 @pytest.mark.remote_data
-def test_save_path_pathlib():
-    pathlib = pytest.importorskip('pathlib')
+@pytest.mark.flaky(reruns=5)
+def test_save_path_pathlib(tmpdir):
     qr = Fido.search(a.Instrument('EVE'), a.Time("2016/10/01", "2016/10/02"), a.Level(0))
 
     # Test when path is pathlib.Path
-    with tempfile.TemporaryDirectory() as target_dir:
-        path = pathlib.Path(target_dir, "{instrument}", "{level}")
-        files = Fido.fetch(qr, path=path)
-        for f in files:
-            assert target_dir in f
-            assert "eve{}0".format(os.path.sep) in f
+    target_dir = tmpdir.mkdir("down")
+    path = pathlib.Path(target_dir, "{instrument}", "{level}")
+    files = Fido.fetch(qr, path=path)
+    for f in files:
+        assert target_dir.strpath in f
+        assert "eve{}0".format(os.path.sep) in f
 
 
 """
@@ -149,7 +153,7 @@ def test_no_time_error():
 @pytest.mark.remote_data
 def test_no_match():
     with pytest.raises(DrmsQueryError):
-        Fido.search(a.jsoc.Time("2016/10/01", "2016/10/02"), a.jsoc.Series("bob"),
+        Fido.search(a.Time("2016/10/01", "2016/10/02"), a.jsoc.Series("bob"),
                     a.vso.Sample(10*u.s))
 
 
@@ -178,17 +182,16 @@ def test_multiple_match():
     with pytest.raises(MultipleMatchError):
         Fido.search(a.Time("2016/10/1", "2016/10/2"), a.Instrument('lyra'))
 
-    Fido.registry = CLIENTS
+    Fido.registry = BaseClient._registry
 
 
 @pytest.mark.remote_data
-def test_no_wait_fetch():
-    qr = Fido.search(a.Instrument('EVE'),
-                     a.Time("2016/10/01", "2016/10/02"),
-                     a.Level(0))
-    res = Fido.fetch(qr, wait=False)
-    assert isinstance(res, DownloadResponse)
-    assert isinstance(res.wait(), list)
+def test_fetch():
+        qr = Fido.search(a.Instrument('EVE'),
+                         a.Time("2016/10/01", "2016/10/02"),
+                         a.Level(0))
+        res = Fido.fetch(qr)
+        assert isinstance(res, Results)
 
 
 """
@@ -301,7 +304,7 @@ def test_fido_iter(queries):
 
 @settings(deadline=50000)
 @given(offline_query())
-def test_repr(query):
+def test_repr2(query):
     res = Fido.search(query)
 
     for rep_meth in (res.__repr__, res.__str__, res._repr_html_):
@@ -312,3 +315,78 @@ def test_repr(query):
         else:
             assert "Provider" not in rep_meth()
             assert "Providers" in rep_meth()
+
+
+@mock.patch("parfive.Downloader.download", return_value=Results(["/tmp/test"]))
+def test_retry(mock_retry):
+    """
+    Test that you can use Fido.fetch to retry failed downloads.
+    """
+    res = Results()
+    res.data.append("/this/worked.fits")
+
+    err1 = FailedDownload("This is not a filename", "http://not.url/test", None)
+    err2 = FailedDownload("This is not a filename2", "http://not.url/test2", None)
+    res.errors.append(err1)
+    res.errors.append(err2)
+
+    mock_retry.return_value._errors += [err2]
+
+    res2 = Fido.fetch(res, Results(["/this/also/worked.fits"]))
+
+    assert res2 is not res
+
+    # Assert that the result of retry ends up in the returned Results() object
+    assert res2.data == ["/this/worked.fits", "/tmp/test", "/this/also/worked.fits", "/tmp/test"]
+    assert res2.errors == [err2, err2]
+
+
+def results_generator(dl):
+    http = list(dl.http_queue._queue)
+    ftp = list(dl.ftp_queue._queue)
+
+    outputs = []
+    for url in http+ftp:
+        outputs.append(pathlib.Path(url.keywords['url'].split("/")[-1]))
+
+    return Results(outputs)
+
+
+@pytest.mark.remote_data
+@mock.patch("sunpy.net.vso.VSOClient.download_all",
+            return_value=Results([], errors=[DownloadFailed(None)]))
+@mock.patch("parfive.Downloader.download", new=results_generator)
+def test_vso_errors_with_second_client(mock_download_all):
+    query = a.Time("2011/01/01", "2011/01/02") & (a.Instrument("goes") | a.Instrument("EIT"))
+
+    qr = Fido.search(query)
+
+    res = Fido.fetch(qr)
+    assert len(res.errors) == 1
+    assert len(res) != qr.file_num
+
+    # Assert that all the XRSClient records are in the output.
+    for resp in qr.responses:
+        if isinstance(resp, XRSClient):
+            assert len(resp) == len(res)
+
+
+def test_downloader_type_error():
+    with pytest.raises(TypeError):
+        Fido.fetch([], downloader=Results())
+
+
+def test_mixed_retry_error():
+    with pytest.raises(TypeError):
+        Fido.fetch([], Results())
+
+
+@mock.patch("sunpy.net.dataretriever.sources.goes.XRSClient.fetch",
+            return_value=["hello"])
+def test_client_fetch_wrong_type(mock_fetch):
+    query = a.Time("2011/01/01", "2011/01/02") & a.Instrument("goes")
+
+    qr = Fido.search(query)
+
+    with pytest.raises(TypeError):
+        Fido.fetch(qr)
